//
//  DecodeViewController.m
//  TXCopyrightedMediaDemo
//
//  Created by Zhirui Ou on 2021/12/6.
//


#define AGORA 1
#define ZEGOLIVEROOM 0

#import "DecodeViewController.h"
#import <UIView+Toast.h>
#import "PcmPlayer.h"
#import <Masonry/Masonry.h>
#import <TXCopyrightedMedia/TXCopyrightedMedia.h>
#import <TXCopyrightedMedia/TXCMAudioFrameInfo.h>
#import <TXCopyrightedMedia/IZegoLiveRoomEngine.h>
#import <TXCopyrightedMedia/IAgoraRtcEngine.h>
#import "ZegoLiveRoomKit.h"
#import "AgoraKit.h"


#if ZEGOLIVEROOM
@interface DecodeViewController ()<TXCMMusicTrackDelegate, TXCMZegoLiveRoomDelegate>
#else
@interface DecodeViewController ()<TXCMMusicTrackDelegate,pcmPlayerDelegate>
#endif
{
    FILE *_captureAudioDumpFile; // 保存回调的音频数据
}

@property (nonatomic, strong) id<ITXCMMusicTrack> musicTrackOrignal; // 原声
@property (nonatomic, strong) id<ITXCMMusicTrack> musicTrackBGM;// 伴奏
@property (nonatomic, strong) id<ITXCMMusicTrack> musicTrackPlaying;// 正在播放的数据源
@property (nonatomic, assign) BOOL orinPrepared; //原唱准备成功
@property (nonatomic, assign) BOOL bgmPrepared; // 伴奏准备成功
@property (nonatomic, strong) TXCMusicInfo *info;
@property (nonatomic, weak) UITextView *activityIdLabel;
@property (nonatomic, weak) UILabel *tipsLabel; // 提示



#if AGORA
@property (nonatomic, strong) AgoraRtcEngineKit *agoraKit;
#endif

#if ZEGOLIVEROOM
@property (nonatomic, strong) ZegoAudioAux *zlrKit;
#endif

@end

@implementation DecodeViewController

- (instancetype)initWithMusicInfo:(TXCMusicInfo *)info
{
    if (self = [self init]) {
        _info = info;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    [self setupUI];
    
#if AGORA
    self.agoraKit = AgoraRtcEngineKit.new;
#endif
    
#if ZEGOLIVEROOM || ZL_PCM || ZL_PCM_SCORE
    self.zlrKit = ZegoAudioAux.new;
#endif
}

- (void)setupUI
{
    self.view.backgroundColor = UIColor.whiteColor;

    UIButton *backButton = [UIButton new];
    [backButton setTitle:@"back" forState:UIControlStateNormal];
    backButton.layer.cornerRadius = 5;
    backButton.backgroundColor = [UIColor purpleColor];
    [backButton addTarget:self action:@selector(backClick) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:backButton];
    
    UIButton *prepareButton = [UIButton new];
    [prepareButton setTitle:@"prepare pcm data" forState:UIControlStateNormal];
    prepareButton.layer.cornerRadius = 5;
    prepareButton.backgroundColor = [UIColor purpleColor];
    [prepareButton addTarget:self action:@selector(prepareClick) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:prepareButton];
        
    UITextView *label = [[UITextView alloc] initWithFrame:CGRectZero];
    label.backgroundColor = [UIColor purpleColor];
    label.textColor = [UIColor whiteColor];
    label.textAlignment = NSTextAlignmentCenter;
    label.layer.cornerRadius = 5;
    label.font = [UIFont systemFontOfSize:14];
    label.text = @"activityId:";
    [self.view addSubview:label];
    self.activityIdLabel = label;
    
    UILabel *tipsLb = [[UILabel alloc] init];
    tipsLb.backgroundColor = [UIColor purpleColor];
    tipsLb.textColor = [UIColor whiteColor];
    tipsLb.textAlignment = NSTextAlignmentCenter;
    tipsLb.numberOfLines = 0;
    tipsLb.layer.cornerRadius = 5;
    tipsLb.font = [UIFont systemFontOfSize:14];
    tipsLb.text = @"msg tips: \n";
    [self.view addSubview:tipsLb];
    self.tipsLabel = tipsLb;
    
    [backButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(20);
        make.top.mas_equalTo(100);
        make.width.mas_equalTo(50);
        make.height.mas_equalTo(30);
    }];
    
    [_tipsLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(25);
        make.right.mas_equalTo(-25);
        make.height.mas_equalTo(100);
        make.top.mas_equalTo(backButton.mas_bottom).offset(40);
    }];
    
//    [_activityIdLabel mas_makeConstraints:^(MASConstraintMaker *make) {
//        make.left.mas_equalTo(25);
//        make.right.mas_equalTo(-25);
//        make.height.mas_equalTo(100);
//        make.top.mas_equalTo(tipsLb.mas_top).offset(-20);
//    }];
    
    [prepareButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(_tipsLabel.mas_width);
        make.height.mas_equalTo(40);
        make.left.mas_equalTo(_tipsLabel.mas_left);
        make.top.mas_equalTo(_tipsLabel.mas_bottom).offset(20);
    }];
}


- (void)dealloc
{
    if (_captureAudioDumpFile != nil) {
        fclose(_captureAudioDumpFile);
    }
    [self.musicTrackOrignal destory];
    self.musicTrackOrignal = nil;
}

#pragma mark - UI Action
- (void)backClick
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)prepareClick
{
    
    self.musicTrackOrignal = [[TXCopyrightedMedia instance] createMusicTrack:self.info];
    [self.musicTrackOrignal setDelegate:self];
    [self.musicTrackOrignal prepare];
    
//    TXCMusicInfo *bgmInfo = [[TXCMusicInfo alloc] init];
//    bgmInfo.musicId = self.info.musicId;
//    bgmInfo.musicType = TXCMusicTypeAccompaniment;
//    bgmInfo.bitrateDefinition = self.info.bitrateDefinition;
//
//    self.musicTrackBGM = [[TXCopyrightedMedia instance] createMusicTrack:bgmInfo];
//    [self.musicTrackBGM setDelegate:self];
//    [self.musicTrackBGM prepare];
    
}


#pragma mark - TXCMMusicTrackDelegate

- (void)onPrepared:(id<ITXCMMusicTrack>)musicTrack
{
    self.tipsLabel.text = @"prepare success。\n 信息输出在xcode控制台!";
#if AGORA
    [musicTrack start];
    AgoraAudioFrame *frame ;
    id<IAgoraRtcEngine> proxy = [self.musicTrackOrignal getProxy:self.agoraKit];
    while (1) {
        TXCMAudioFrameInfo *info = [self.musicTrackOrignal readAudioFrame];
        if (info == nil) {
            break;
        }
        int channelCount = [self.musicTrackOrignal getChannelCount];
        // mp3:1152 m4a:1024
        int perChannel = [self.musicTrackOrignal getFramePerPacket];
        
        [proxy pushExternalAudioFrameRawData:info
                                   sourcePos:0
                           samplesPerChannel:perChannel
                              bytesPerSample:1
                                    channels:channelCount
                               samplesPerSec:1
                                renderTimeMs:0
                                  avSyncType:0];
    }
    NSString* systemDocumentPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) firstObject];
    NSString* filePath = [NSString stringWithFormat:@"%@/%@", systemDocumentPath, @"agora.pcm"];
    self.tipsLabel.text = [NSString stringWithFormat:@"数据输出路径:\n%@",filePath];

#elif ZEGOLIVEROOM || ZL_PCM_SCORE
    // 只演示一下原唱
    if (musicTrack != self.musicTrackOrignal) {
        return;
    }
    [musicTrack start];
    id<IZegoLiveRoomEngine> proxy = [self.musicTrackOrignal getProxy:self.zlrKit];
    [proxy setZegoLiveRoomDelegate:self];
#endif
}


- (void)onError:(id<ITXCMMusicTrack>)musicTrack errCode:(int)errCode msg:(NSString *)msg
{
    NSLog(@"decodeError : %@",msg);
    
    self.tipsLabel.text = [NSString stringWithFormat:@"prepare failed :\n%@",msg];
}

#pragma mark - TXCMZegoLiveRoomDelegate
#if ZEGOLIVEROOM
- (TXCMAudioFrameInfo *)onLiveRoomAuxCallback:(id<IZegoLiveRoomEngine>)zegoProxy
{
    TXCMAudioFrameInfo *info = [self.musicTrackOrignal readAudioFrame];
    if (info != nil) {
        NSLog(@"onLiveRoomAuxCallback timestamp = %@", @(info.timestamp));
    }else {
        id<IZegoLiveRoomEngine> proxy = [self.musicTrackOrignal getProxy:self.zlrKit];
        [proxy setZegoLiveRoomDelegate:nil];
        [self.musicTrackOrignal destory];
        self.musicTrackOrignal = nil;
    }
    return info;
}
#endif

@end




