//
//  SongScoreViewController.m
//  TXCopyrightedMediaDemo
//
//  Created by Zhirui Ou on 2022/1/13.
//

#import "SongScoreViewController.h"
#import <Masonry/Masonry.h>
#import <TXCopyrightedMedia/TXCopyrightedMedia.h>


static const int kSampleRate = 44100;
static const int kChannelCount = 1;
static const int kBuffLength = 2048;

@interface SongScoreViewController ()<TXCSongScoreDelegate>

@property (nonatomic, strong) id<ITXCSongScore> songScore;
@property (nonatomic, strong) NSFileHandle *pcmFileHandle;
@property (nonatomic, weak) UITextView *messsageLabel;

@end

@implementation SongScoreViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.view.backgroundColor = UIColor.whiteColor;
    
    UIButton *backButton = [UIButton new];
    [backButton setTitle:@"back" forState:UIControlStateNormal];
    backButton.layer.cornerRadius = 5;
    backButton.backgroundColor = [UIColor purpleColor];
    [backButton addTarget:self action:@selector(backClick) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:backButton];
    
    [backButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(20);
        make.top.mas_equalTo(100);
        make.width.mas_equalTo(50);
        make.height.mas_equalTo(30);
    }];
    
    UIButton *playBtn = [UIButton buttonWithType:UIButtonTypeSystem];
    playBtn.frame = CGRectMake((self.view.frame.size.width - 150) / 2 , 200, 150, 50);
    playBtn.layer.cornerRadius = 25;
    playBtn.layer.masksToBounds = YES;
    playBtn.backgroundColor = [UIColor colorWithRed:0x1E/255.0 green:0x90/255.0 blue:0xFF/255.0 alpha:1.0];
    [playBtn setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
    [playBtn setTitle:@"Start" forState:UIControlStateNormal];
    playBtn.titleLabel.font = [UIFont systemFontOfSize:20.0 weight:UIFontWeightBold];
    [playBtn addTarget:self action:@selector(play:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:playBtn];
    
    
    UITextView *textView = [[UITextView alloc] init];
    textView.frame = CGRectMake(50, CGRectGetMaxY(playBtn.frame) + 50, CGRectGetWidth(self.view.frame) - 100, 200);
    textView.layer.borderColor = [UIColor lightGrayColor].CGColor;
    textView.layer.borderWidth = 1.0;
//    label.numberOfLines = 0;
    [self.view addSubview:textView];
    self.messsageLabel = textView;
}

- (void)dealloc
{
    if (self.songScore) {
        [self.songScore destory];//销毁
    }
}

#pragma mark - action
- (void)backClick
{
    [self dismissViewControllerAnimated:YES completion:nil];
}


- (void)play:(UIButton *)button {
    self.messsageLabel.text = @"开始打分";

    NSString *noteFilePath = [[NSBundle mainBundle] pathForResource:@"ysd_midi" ofType:@"txt"];
    NSString *lyricFilePath = [[NSBundle mainBundle] pathForResource:@"ysd_lyric" ofType:@"vtt"];
    
    TXCSongScoreConfig *config = [[TXCSongScoreConfig alloc] init];
    config.sampleRate = kSampleRate;//录音文件采样率
    config.channel = kChannelCount;//录音文件声道数
    config.noteFilePath = noteFilePath;//音准文件路径
    config.lyricFilePath = lyricFilePath;//歌词文件路径
    self.songScore = [[TXCopyrightedMedia instance] createSongScore:config];//创建打分类
    self.songScore.delegate = self;//设置代理回调

    NSString *pcmFilePath = [[NSBundle mainBundle] pathForResource:@"ysd_sing" ofType:@"pcm"];
    self.pcmFileHandle = [NSFileHandle fileHandleForReadingAtPath:pcmFilePath];
    
    [self.songScore prepare];//初始化准备
}

#pragma mark - TXCSongScoreDelegate

- (void)songScoreOnPrepared:(id<ITXCSongScore>)songScore
{
    __block NSInteger sampleOffset = 0;
    __block NSError *seekError = nil;
//    [self.songScore seek:sampleOffset];
        [self.pcmFileHandle seekToOffset:sampleOffset error:&seekError];
        while (YES) {
            NSData *bufferData = [self.pcmFileHandle readDataOfLength:kBuffLength];
            if (bufferData.length <= 0) {
                [self.songScore finish];
                break;
            }
            double timePosition = (double)sampleOffset / kSampleRate / 2 * 1000.0;
            [self.songScore process:(char *)bufferData.bytes length:(int)bufferData.length timeStamp:timePosition];//输入buff
            sampleOffset += bufferData.length;
        };
    
}

- (void)songScoreOnError:(id<ITXCSongScore>)songScore
                 errCode:(int)errCode
                     msg:(NSString *)msg
{
    NSLog(@"errCode = %@, msg = %@", @(errCode), msg);
    self.messsageLabel.text = [NSString stringWithFormat:@"错误! Code = %@, msg = %@",@(errCode), msg];
}

- (void)songScoreDidUpdate:(id<ITXCSongScore>)songScore
              currentScore:(int)currentScore
                totalScore:(int)totalScore
                  curIndex:(int)curIndex
{
    NSLog(@"current = %d, totalScore = %d, curIndex = %d", currentScore, totalScore, curIndex);
}

- (void)songScoreDidFinish:(id<ITXCSongScore>)songScore
                scoreArray:(NSArray *)scoreArray
                totalScore:(int)totalScore
{
    NSLog(@"score array = %@, totalScore = %d", scoreArray, totalScore);
    dispatch_async(dispatch_get_main_queue(), ^{
        self.messsageLabel.text = [NSString stringWithFormat:@"每句打分结果 = %@ \n 总得分 = %d",scoreArray, totalScore];
    });
}

- (void)songScoreDidGroveAndHit:(id<ITXCSongScore>)songScore
                      timeStamp:(double)timeStamp
                          isHit:(BOOL)isHit
                          pitch:(float)pitch
                      viewValue:(int)viewValue
{
    NSLog(@"timeStamp = %@, isHit = %d, userPitch = %f, viewValue = %d", @(timeStamp), isHit, pitch, viewValue);
}



/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
