//
//  ViewController.m
//  TXCopyrightedMediaDemo
//
//  Created by Zhirui Ou on 2021/5/14.
//

#import "ViewController.h"

#import <TXCopyrightedMedia/TXCopyrightedMedia.h>
#import "DecodeViewController.h"
#import "SongScoreViewController.h"

#import <Masonry/Masonry.h>
#import "TXCopyrightHTTPManager.h"
#import <AVFoundation/AVFoundation.h>
#import <AVKit/AVKit.h>
#import <UIView+Toast.h>

// 从后台获取到的音乐id
static NSString *const musicId = @"<#从后台获取到的音乐id#>";
// 从后台获取到的musicId对应的token
static NSString *token = @"<#musicId对应的Token#>";

@interface ViewController ()<ITXMusicPreloadCallback, ITXMusicDownloadLicenseCallback>
{
    id _playTimeObserver;
}

@property (nonatomic, weak) AVPlayerViewController *playerVC;
@property (nonatomic, strong) AVPlayer *player;
@property (nonatomic, strong) AVPlayerItem *playerItem;
@property (nonatomic, strong) NSDate *startDate;
@property (nonatomic, assign) NSInteger testTotal;
@property (nonatomic, assign) NSTimeInterval totalTime;

@property (nonatomic, strong) UITextField *musicIdtf;
@property (nonatomic, strong) UITextField *tokentf;
@property (nonatomic, strong) UILabel *preloadRetLabel;
@property (nonatomic, strong) UITextView *genUriLabel;

// config version
@property (nonatomic, assign) BOOL isYsdVersion;
@property (nonatomic, copy) NSString *defaultBiteRate;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // 从腾讯云申请的licenseUrl 和 Key
    NSString *licence = @"<#license url#>";
    NSString *key = @"<#license key#>";
    
    [[TXCopyrightedMedia instance] setLicense:licence key:key callback:self];
    [[TXCopyrightedMedia instance] initialization];
    [[TXCopyrightedMedia instance] setMusicCacheMaxCount:10];
    [TXCopyrightedMedia enableLog:NO];
    
    [self configVersion];
    NSLog(@"%@", NSHomeDirectory());
    
    [self setupUI];
}

- (void)configVersion
{
    NSString *version = [TXCopyrightedMedia getSDKVersion];
    NSLog(@"---- sdk version : %@",version);
    self.isYsdVersion = [version containsString:@"ysd"];
    self.defaultBiteRate = @"audio/default";
    if (!self.isYsdVersion) {
        // yjx版本有三个码率 audio/lo audio/mi audio/hi
        // ysd 填默认的 audio/default
        self.defaultBiteRate = @"audio/lo";
    }
    
}

- (void)setupUI
{
    UITextField *musicIdtf = UITextField.new;
    musicIdtf.placeholder = @"请输入musicId";
    musicIdtf.text = musicId;
    musicIdtf.borderStyle = UITextBorderStyleRoundedRect;
    [self.view addSubview:musicIdtf];
    self.musicIdtf = musicIdtf;
    
    UILabel *musicLb = [[UILabel alloc] init];
    musicLb.text = @"musicId:";
    musicLb.font = [UIFont systemFontOfSize:14];
    [self.view addSubview:musicLb];
    
    UILabel *tokenLb = [[UILabel alloc] init];
    tokenLb.text = @"playToken:";
    tokenLb.font = [UIFont systemFontOfSize:14];
    [self.view addSubview:tokenLb];
    
    UITextField *tokentf = UITextField.new;
    tokentf.placeholder = @"请输入token";
    tokentf.borderStyle = UITextBorderStyleRoundedRect;
    tokentf.text = token;
    [self.view addSubview:tokentf];
    self.tokentf = tokentf;
    
    UIButton *preloadButton = [UIButton new];
    [preloadButton setTitle:@"Preload Music" forState:UIControlStateNormal];
    preloadButton.backgroundColor = [UIColor purpleColor];
    preloadButton.layer.cornerRadius = 5;
    [preloadButton addTarget:self action:@selector(preloadClick) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:preloadButton];
    
    UILabel *preloadLabel = [[UILabel alloc] init];
    preloadLabel.text = @" Preload Reslut:\n";
    preloadLabel.numberOfLines = 2;
    preloadLabel.layer.cornerRadius = 5;
    preloadLabel.layer.borderColor = [UIColor lightGrayColor].CGColor;
    preloadLabel.layer.borderWidth = 1.0;
    preloadLabel.font = [UIFont systemFontOfSize:14];
    [self.view addSubview:preloadLabel];
    self.preloadRetLabel = preloadLabel;
    
    UIButton *clearButton = [UIButton new];
    [clearButton setTitle:@"Clear Cache" forState:UIControlStateNormal];
    clearButton.backgroundColor = [UIColor purpleColor];
    clearButton.layer.cornerRadius = 5;
    [clearButton addTarget:self action:@selector(clearCache) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:clearButton];
    
    UIButton *bgmUriGetButton = [UIButton new];
    [bgmUriGetButton setTitle:@"Gen Uri" forState:UIControlStateNormal];
    bgmUriGetButton.backgroundColor = [UIColor purpleColor];
    bgmUriGetButton.layer.cornerRadius = 5;
    [bgmUriGetButton addTarget:self action:@selector(genBGMURI) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:bgmUriGetButton];
    
//    UILabel *genUriLabel = [[UILabel alloc] init];
    UITextView *genUriLabel = [[UITextView alloc] initWithFrame:CGRectZero];
    genUriLabel.text = @" Gen Reslut:\n";
//    genUriLabel.numberOfLines = 0;
    genUriLabel.layer.cornerRadius = 5;
    genUriLabel.layer.borderColor = [UIColor lightGrayColor].CGColor;
    genUriLabel.layer.borderWidth = 1.0;
    genUriLabel.font = [UIFont systemFontOfSize:12];
    [self.view addSubview:genUriLabel];
    self.genUriLabel = genUriLabel;
    
    UIButton *decodeButton = [UIButton new];
    [decodeButton setTitle:@"Output Pcm And Play" forState:UIControlStateNormal];
    decodeButton.backgroundColor = [UIColor purpleColor];
    decodeButton.layer.cornerRadius = 5;
    [decodeButton addTarget:self action:@selector(testDecoder) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:decodeButton];
    [decodeButton sizeToFit];
    
    UIButton *scoreButton = [UIButton new];
    [scoreButton setTitle:@"Score" forState:UIControlStateNormal];
    scoreButton.backgroundColor = [UIColor purpleColor];
    scoreButton.layer.cornerRadius = 5;
    [scoreButton addTarget:self action:@selector(songScore) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:scoreButton];
    [scoreButton sizeToFit];
    
    [musicIdtf mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view).with.offset(32);
        make.right.equalTo(self.view).with.offset(-32);
        make.top.equalTo(self.view.mas_top).offset(100);
    }];
    
    [musicLb mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(musicIdtf.mas_left);
        make.bottom.mas_equalTo(musicIdtf.mas_top).offset(-5);
    }];
    
    [tokentf mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view).with.offset(32);
        make.right.equalTo(self.view).with.offset(-32);
        make.top.equalTo(musicIdtf.mas_bottom).with.offset(34);
    }];
    
    [tokenLb mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(tokentf.mas_left);
        make.bottom.mas_equalTo(tokentf.mas_top).offset(-5);
    }];
    
    [preloadButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(tokentf.mas_bottom).offset(20);
        make.height.mas_equalTo(40);
        make.width.mas_equalTo(tokentf.mas_width).multipliedBy(0.5).offset(-10);
        make.left.mas_equalTo(tokentf.mas_left);
    }];
    
    [clearButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(tokentf.mas_bottom).offset(20);
        make.height.mas_equalTo(40);
        make.width.mas_equalTo(tokentf.mas_width).multipliedBy(0.5).offset(-10);
        make.right.mas_equalTo(tokentf.mas_right);
    }];
    
    [preloadLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(preloadButton.mas_left);
        make.right.mas_equalTo(clearButton.mas_right);
        make.top.mas_equalTo(preloadButton.mas_bottom).offset(10);
        make.height.mas_equalTo(50);
    }];
    
    [bgmUriGetButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(preloadLabel.mas_bottom).offset(15);
        make.left.mas_equalTo(preloadLabel.mas_left);
        make.height.mas_equalTo(40);
        make.right.mas_equalTo(clearButton.mas_right);
    }];
    
    [genUriLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(preloadButton.mas_left);
        make.right.mas_equalTo(clearButton.mas_right);
        make.top.mas_equalTo(bgmUriGetButton.mas_bottom).offset(10);
        make.height.mas_equalTo(180);
    }];
    
    [decodeButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(preloadButton.mas_left);
        make.right.mas_equalTo(clearButton.mas_right);
        make.top.equalTo(genUriLabel.mas_bottom).offset(15);
        make.height.mas_equalTo(40);
    }];
    
    [scoreButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(preloadButton.mas_left);
        make.right.mas_equalTo(clearButton.mas_right);
        make.top.equalTo(decodeButton.mas_bottom).offset(15);
        make.height.mas_equalTo(40);
    }];
}

#pragma mark - action

- (void)preloadClick
{
    [[TXCopyrightedMedia instance] preloadMusic:self.musicIdtf.text bitrateDefinition:self.defaultBiteRate playToken:self.tokentf.text callback:self];
}

- (void)clearCache
{
    [[TXCopyrightedMedia instance] clearMusicCache];
    self.preloadRetLabel.text = @"清理缓存成功...";
}

- (void)genBGMURI
{
    NSString *orignalUri = [[TXCopyrightedMedia instance] genMusicURI:self.musicIdtf.text bgmType:0 bitrateDefinition:self.defaultBiteRate];
    NSString *accompanimentUri = [[TXCopyrightedMedia instance] genMusicURI:self.musicIdtf.text bgmType:1 bitrateDefinition:self.defaultBiteRate];
    NSString *lyricsUri = [[TXCopyrightedMedia instance] genMusicURI:self.musicIdtf.text bgmType:2 bitrateDefinition:self.defaultBiteRate];
    
    NSString *result = [NSString stringWithFormat:@"原唱URL : %@\n 伴奏URL : %@\n 歌词Uri : %@\n",orignalUri,accompanimentUri,lyricsUri];
    self.genUriLabel.text = result;
}

- (void)testDecoder
{
    TXCMusicInfo *info = TXCMusicInfo.new;
    info.musicId = self.musicIdtf.text;
    info.bitrateDefinition = self.defaultBiteRate;
    info.musicType = 0;
    DecodeViewController *vc = [[DecodeViewController alloc] initWithMusicInfo:info];
    UINavigationController *nac = [[UINavigationController alloc] initWithRootViewController:vc];
    nac.modalPresentationStyle = UIModalPresentationOverFullScreen;
    [self presentViewController:nac animated:YES completion:nil];
}

- (void)songScore
{
    UINavigationController *nac = [[UINavigationController alloc] initWithRootViewController:SongScoreViewController.new];
    nac.modalPresentationStyle = UIModalPresentationOverFullScreen;
    [self presentViewController:nac animated:YES completion:nil];
}

#pragma mark - ITXBGMPreloadCallback

- (void)onPreloadStart:(NSString *)musicId bitrateDefinition:(NSString *)bitrateDefinition
{
    [self.view endEditing:YES];
    [self.view makeToast:@"开始下载"];
    self.startDate = NSDate.date;
    NSLog(@"onPreloadStart %@", musicId);
    self.preloadRetLabel.text = [NSString stringWithFormat:@" Start Preload..."];
}

- (void)onPreloadProgress:(NSString *)musicId bitrateDefinition:(NSString *)bitrateDefinition progress:(float)progress
{
    NSLog(@"onPreloadProgress %@ %@", musicId, @(progress));
    [self.view hideToast];
    [self.view makeToast:[NSString stringWithFormat:@"进度:%d", (int)(progress * 100)]];
    self.preloadRetLabel.text = [NSString stringWithFormat:@" Preload Progress: \n %d",(int)(progress * 100)];
}

- (void)onPreloadComplete:(NSString *)musicId bitrateDefinition:(NSString *)bitrateDefinition errorCode:(int)errorCode msg:(NSString *)msg
{
    if (errorCode == 0) {
        [self.view makeToast:@"加载完成"];
    }else {
        [self.view makeToast:[NSString stringWithFormat:@"加载失败:%@",msg]];
    }
    
    self.preloadRetLabel.text = [NSString stringWithFormat:@" Preload Result : %d %@",errorCode,msg];
    
    NSLog(@"errorCode = %@, cost time = %@", @(errorCode), @(NSDate.date.timeIntervalSince1970 - self.startDate.timeIntervalSince1970));
}

#pragma mark - ITXMusicSetLicenseCallback

/// setLicense完成时回调
/// @param errorCode 内部错误码，0表示没有错误
/// @param msg 错误信息
- (void)onDownloadLicenseComplete:(int)errorCode
                              msg:(NSString *)msg
{
    NSLog(@"onDownloadLicenseComplete code:%d %@",errorCode,msg);
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    [self.view endEditing:YES];
}

@end
